@@ -1,8 +1,8 @@
 #!/bin/sh
- 
-# default setting
-PGSRC=/usr/local/src/postgres/postgresql-7.4beta3
 
+# default setting
+PGSRC=/usr/local/src/postgres/postgresql-7.3.6
+ 
 # check user
 if [ `id -u` -ne 0 ];then
 	echo 'This command in only executed by root'
@@ -35,13 +35,13 @@ fi
 echo
 echo "PGSRC is setted to ${PGSRC}"
 echo 'setting is ok ? continue setting ?'
-select input in y n
+select input in yes no
 do
 	case $input in
-	    'y')
+	    'yes')
 			break
 			;;
-	    'n')
+	    'no')
 	    	echo
 	        echo 'quit'
 	        exit 1;
@@ -54,29 +54,11 @@ do
 	esac
 done
 
-TARGET=$PGSRC/contrib/intarray
+TARGET=${PGSRC}/contrib/intarray
 
-echo
-echo 'mv old intarray to intarray.old'
-rm -rf ${PGSRC}/contrib/intarray.old
-mv -v ${PGSRC}/contrib/intarray ${PGSRC}/contrib/intarray.old
+echo "copy files to ${TARGET}"
+cp _int.c _int_add.sql create.sh ${TARGET}
 
-echo
-echo 'now extracting..'
-tar xzf intarray.tar.gz
-echo
-
-echo "mv new intarray to ${PGSRC}/contrib"
-mv intarray ${PGSRC}/contrib
-echo
-
-echo 'prepared !'
-
-sleep 3
-
-echo
-echo 'start setting database'
-echo
 ./create.sh ${TARGET}
 
-echo 'finish !'
\ No newline at end of file
+echo 'finish !'
