@@ -5,7 +5,8 @@ SELECT 'DROP INDEX xml_node_next' as phase2;
 DROP INDEX "xml_node_next";
 
 -- if repository is a static.
-SELECT 'CREATE INDEX for xml_node.dewey' as phase3;
-CREATE INDEX xml_node_dewey ON xml_node USING btree(dewey intarray_abs_ops);
+SELECT 'CREATE INDEX for xml_node_dewey' as phase3;
+REINDEX INDEX "xml_node_dewey";
+--CREATE INDEX xml_node_dewey ON xml_node USING btree(dewey intarray_abs_ops);
 
-ANALYZE;
\ No newline at end of file
+VACUUM ANALYZE;
