@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  */
 
--- This function is yet beta stage now. created one day..(-:
+-- This function is yet beta stage now. created a half of day..(-:
 -- not tested well. report of testing is welcome.
 -- send problems to mailing-list.
 
@@ -45,39 +45,38 @@
 //[p] : partical
 
 -- DOMImplementation
-[-] createDocument() ex [DOMImplementation.createDocument()]
-[n] createDocumentType() ex [DOMImplementation.createDocumentType()]
-[y] hasFeature() ex [DOMImplementation.hasFeature()]
+[-] createDocument() [n] createDocumentType()
+[y] hasFeature()
 
 -- Document
-[y] createAttribute() ex [Document.createAttribute()]
-[-] createAttributeNS() ex [Document.createAttributeNS()]
-[n] createCDATASection() ex [Document.createCDATASection()]
-[y] createComment() ex [Document.createComment()]
-[-] createDocumentFragment() ex [Document.createDocumentFragment()]
-[y] createElement() ex [Document.createElement()]
-[y] createElementNS() ex [Document.createElementNS()]
-[n] createEntityReference() ex [Document.createEntityReference()]
-[y] createProcessingInstruction() ex [Document.createProcessingInstruction()]
-[y] createTextNode() ex [Document.createTextNode()]
-[n] getElementById() ex [Document.getElementById()]
-[y] getElementsByTagName() ex [Document.getElementsByTagName()]
-[-] getElementsByTagNameNS() ex [Document.getElementsByTagNameNS()]
-[-] importNode() ex [Document.importNode()]
+[y] createAttribute() 
+[-] createAttributeNS() 
+[n] createCDATASection()
+[y] createComment() 
+[y] createDocumentFragment()
+[y] createElement() 
+[y] createElementNS() 
+[n] createEntityReference() 
+[y] createProcessingInstruction() 
+[y] createTextNode() 
+[n] getElementById() 
+[y] getElementsByTagName()
+[-] getElementsByTagNameNS() 
+[-] importNode()
 
 -- Node
-[y] appendChild() ex [Node.appendChild()]
-[-] cloneNode() ex [Node.cloneNode()]
-[y] hasAttributes() ex [Node.hasAttributes()]
-[y] hasChildNodes() ex [Node.hasChildNodes()]
-[y] insertBefore() ex [Node.insertBefore()]
-[n] isSupported() ex [Node.isSupported()]
-[-] normalize() ex [Node.normalize()]
-[-] removeChild() ex [Node.removeChild()]
-[-] replaceChild() ex [Node.replaceChild()]
+[y] appendChild() 
+[-] cloneNode()
+[y] hasAttributes() 
+[y] hasChildNodes() 
+[y] insertBefore()
+[n] isSupported() 
+[-] normalize() 
+[n] removeChild() 
+[-] replaceChild()
 
 -- NodeList
-[n] item() ex [NodeList.item()]
+[n] item()
 
 -- NamedNodeMap
 [n] getNamedItem() ex [NamedNodeMap.getNamedItem()]
@@ -155,6 +154,12 @@ TODO:
 
 */
 
+--CREATE OR REPLACE FUNCTION plpgsql_call_handler () RETURNS LANGUAGE_HANDLER AS
+--    '$libdir/plpgsql' LANGUAGE C;
+--
+--CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql
+--    HANDLER plpgsql_call_handler;
+
 --------------------------------------------------------------------------
 -- Private functions
 --------------------------------------------------------------------------
@@ -168,6 +173,7 @@ AS '
 			WHEN ''TEXT'' THEN 2
 			WHEN ''ATTRIBUTE'' THEN 5
 			WHEN ''NAMESPACE'' THEN 6
+			WHEN ''DOCUMENT'' THEN 0
 			WHEN ''ROOT'' THEN 0
 			WHEN ''DUMMY'' THEN -1
 			WHEN ''COMMENT'' THEN 3
@@ -211,6 +217,28 @@ CREATE OR REPLACE FUNCTION getDocumentElement (INTEGER) RETURNS INTEGER AS '
 	    xn2.parent = xn1.id
 ' LANGUAGE 'sql' STABLE STRICT;
 
+/**
+ -- dropDocument(docname VARCHAR)::Boolean
+ 	drop the specified document
+ */
+CREATE OR REPLACE FUNCTION dropDocument (VARCHAR) RETURNS BOOLEAN AS '
+	DECLARE
+		V_DOCNAME ALIAS FOR $1;
+		V_DOCID INTEGER;
+		MVIEWS RECORD;
+	BEGIN
+		SELECT INTO V_DOCID docid FROM xml_document WHERE name = V_DOCNAME;
+		IF NOT FOUND THEN
+			RAISE NOTICE ''Document % not found'',V_DOCNAME;
+			RETURN FALSE;
+		END IF;	
+		-- delete document.
+		DELETE FROM xml_document WHERE docid = V_DOCID;
+		DELETE FROM xml_node WHERE docid = V_DOCID;
+		RETURN TRUE;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+
 --------------------------------------------------------------------------
 -- Interface DOMImplementation
 --------------------------------------------------------------------------
@@ -286,9 +314,125 @@ CREATE OR REPLACE FUNCTION createAttribute (INTEGER, VARCHAR) RETURNS INTEGER AS
  -- XPath ignores CDATA
 
 /**
- -- Document.createDocumentFragment()
+ -- createComment(Document INTEGER, Data VARCHAR)
  */
--- not yet supported
+CREATE OR REPLACE FUNCTION createComment (INTEGER, VARCHAR) RETURNS INTEGER AS '
+	DECLARE
+		V_DOCNODE ALIAS FOR $1;
+		V_DATANAME ALIAS FOR $2;
+		V_DOCID INTEGER;
+		V_NODEID INTEGER;
+		V_DEWEY INTEGER[];
+		V_PARENT INTEGER;
+	BEGIN
+		RAISE DEBUG ''createComment(%, %)'',V_DOCNODE,V_DATANAME;
+		-- gets document id.
+		SELECT INTO V_DOCID docid 
+			FROM xml_node
+			WHERE id = V_DOCNODE AND kind = getNodeType(''DOCUMENT'');
+		IF NOT FOUND THEN
+			RAISE EXCEPTION ''Raising a DOMException, first argument must be document-node.'';
+		END IF;
+		V_NODEID := nextval(''xml_nodeid_seq'');
+		-- For the backward compatibility.
+		-- to be appeared
+		-- Supported from PostgreSQL 7.4
+		V_DEWEY := ARRAY[-1, V_DOCID, nextval(''xml_node_dummy_seq'')];
+		SELECT INTO V_PARENT id
+			FROM xml_node WHERE docid = V_DOCID AND kind = getNodeType(''DUMMY'');
+		-- inserts a node.
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey,parent,value)
+			VALUES(V_NODEID,getNodeType(''COMMNET''),V_DOCID,V_NODEID,V_DEWEY,V_PARENT,V_DATANAME);
+		INSERT INTO xml_path VALUES(V_NODEID,''#/comment()'',''false'');
+		RAISE DEBUG ''Created Comment..%d'',V_NODEID;
+		RETURN V_NODEID;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+
+/**
+ -- Document.createDocumentFragment()
+ -- proprietary extention
+ --
+ -- Description:  
+ --		Creates an empty DocumentFragment object. 
+ --
+ -- Returns:  
+ --		DocumentFragment -  A new DocumentFragment. 
+ --
+ -- This function does not fit to the specification.
+ -- have additional argument.
+ --
+ -- This function is supported postgres 7.4 or later. 
+ */
+CREATE OR REPLACE FUNCTION createDocumentFragment (VARCHAR) RETURNS INTEGER AS '
+	DECLARE
+		V_DOCNAME ALIAS FOR $1;
+		V_DOCID INTEGER;
+		V_NODEID INTEGER;
+		V_DEWEY INTEGER[];
+	BEGIN
+		SELECT INTO V_DOCID docid
+			FROM xml_document WHERE name = V_DOCNAME;
+		IF FOUND THEN
+			RETURN NULL;
+		END IF;
+		SELECT INTO V_DOCID 
+			coalesce(max(docid)+1,1) FROM xml_document;
+		INSERT INTO xml_document(docid,name) VALUES(V_DOCID,V_DOCNAME);
+		-- create parent of root node, document flagment.
+		V_NODEID := nextval(''xml_nodeid_seq'');
+		PERFORM setval(''xml_nodeid_seq'', V_NODEID + 1);
+		-- PostgreSQL 7.4
+		-- V_DEWEY := ARRAY[0, V_DOCID];
+		-- PostgreSQL 7.3
+		V_DEWEY := ''{0}''::_int4 + V_DOCID;
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey)
+			VALUES(V_NODEID, getNodeType(''ROOT''), V_DOCID, 0, V_DEWEY);
+		-- create dummy root node
+		-- V_DEWEY := ARRAY[-1, V_DOCID];
+		V_DEWEY := ''{-1}''::_int4 + V_DOCID;
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey)
+			VALUES(V_NODEID + 1, getNodeType(''DUMMY''), V_DOCID, 0, V_DEWEY);
+		RETURN V_NODEID;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+
+/**
+ -- createElement(Document INTEGER, tagName VARCHAR)::INTEGER
+ -- returns new created node id.
+ */
+CREATE OR REPLACE FUNCTION createElement (INTEGER, VARCHAR) RETURNS INTEGER AS '
+	DECLARE
+		V_DOCNODE ALIAS FOR $1;
+		V_TAGNAME ALIAS FOR $2;
+		V_DOCID INTEGER;
+		V_NODEID INTEGER;
+		V_PARENT INTEGER;
+		V_DEWEY INTEGER[];
+	BEGIN
+		RAISE DEBUG ''createElement(%, %)'',V_DOCNODE,V_TAGNAME;
+		-- gets document id.
+		SELECT INTO V_DOCID docid 
+			FROM xml_node
+			WHERE id = V_DOCNODE AND kind = getNodeType(''DOCUMENT'');
+		IF NOT FOUND THEN
+			RAISE EXCEPTION ''Raising a DOMException, first argument must be document-node.'';
+		END IF;
+		V_NODEID := nextval(''xml_nodeid_seq'');
+		-- For the backward compatibility.
+		-- to be appeared.
+		-- Supported from PostgreSQL 7.4
+		V_DEWEY := ARRAY[-1, V_DOCID, nextval(''xml_node_dummy_seq'')];
+		SELECT INTO V_PARENT id
+			FROM xml_node WHERE docid = V_DOCID AND kind = getNodeType(''DUMMY'');
+		-- inserts a node.
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey,parent,tagid)
+			VALUES(V_NODEID,getNodeType(''ELEMENT''),V_DOCID,V_NODEID,V_DEWEY,V_PARENT,xml_tagid(V_TAGNAME));
+		INSERT INTO xml_path VALUES(V_NODEID,''#/'' || V_TAGNAME,''false'');
+		RAISE DEBUG ''Created Element..%d'',V_NODEID;
+		RETURN V_NODEID;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
 
 /**
  -- createElementNS(Document INTEGER, namespaceURI VARCHAR, qualifiedName VARCHAR)::INTEGER
@@ -332,6 +476,79 @@ CREATE OR REPLACE FUNCTION createElementNS (INTEGER, VARCHAR, VARCHAR) RETURNS I
  */
 -- no plan to support now.
 
+/**
+ -- createProcessingInstruction(Document INTEGER, Target VARCHAR, Data VARCHAR)::INTEGER
+ */
+CREATE OR REPLACE FUNCTION createProcessingInstruction (INTEGER, VARCHAR, VARCHAR) RETURNS INTEGER AS '
+	DECLARE
+		V_DOCNODE ALIAS FOR $1;
+		V_TARGET ALIAS FOR $2;
+		V_DATANAME ALIAS FOR $3;
+		V_DOCID INTEGER;
+		V_NODEID INTEGER;
+		V_DEWEY INTEGER[];
+		V_PARENT INTEGER;
+	BEGIN
+		RAISE DEBUG ''createProcessingInstruction(%, %, %)'',V_DOCNODE,V_TARGET,V_DATANAME;
+		-- gets document id.
+		SELECT INTO V_DOCID docid 
+			FROM xml_node
+			WHERE id = V_DOCNODE AND kind = getNodeType(''DOCUMENT'');
+		IF NOT FOUND THEN
+			RAISE EXCEPTION ''Raising a DOMException, first argument must be document-node.'';
+		END IF;
+		V_NODEID := nextval(''xml_nodeid_seq'');
+		-- For the backward compatibility.
+		-- to be appeared
+		-- Supported from PostgreSQL 7.4
+		V_DEWEY := ARRAY[-1, V_DOCID, nextval(''xml_node_dummy_seq'')];
+		SELECT INTO V_PARENT id
+			FROM xml_node WHERE docid = V_DOCID AND kind = getNodeType(''DUMMY'');
+		-- inserts a node.
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey,parent,tagid,value)
+			VALUES(V_NODEID,getNodeType(''PI''),V_DOCID,V_NODEID,V_DEWEY,V_PARENT,xml_piid(V_TARGET),V_DATANAME);
+		INSERT INTO xml_path VALUES(V_NODEID,''#/processing-instruction('' || V_TARGET || '')'',''false'');
+		RAISE DEBUG ''Created PI..%d'',V_NODEID;
+		RETURN V_NODEID;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+
+/**
+ -- createTextNode(Document INTEGER, Data VARCHAR)
+ */
+CREATE OR REPLACE FUNCTION createTextNode (INTEGER, VARCHAR) RETURNS INTEGER AS '
+	DECLARE
+		V_DOCNODE ALIAS FOR $1;
+		V_DATANAME ALIAS FOR $2;
+		V_DOCID INTEGER;
+		V_NODEID INTEGER;
+		V_DEWEY INTEGER[];
+		V_PARENT INTEGER;
+	BEGIN
+		RAISE DEBUG ''createTextNode(%, %)'',V_DOCNODE,V_DATANAME;
+		-- gets document id.
+		SELECT INTO V_DOCID docid 
+			FROM xml_node
+			WHERE id = V_DOCNODE AND kind = getNodeType(''DOCUMENT'');
+		IF NOT FOUND THEN
+			RAISE EXCEPTION ''Raising a DOMException, first argument must be document-node.'';
+		END IF;
+		V_NODEID := nextval(''xml_nodeid_seq'');
+		-- For the backward compatibility.
+		-- to be appeared
+		-- Supported from PostgreSQL 7.4
+		V_DEWEY := ARRAY[-1, V_DOCID, nextval(''xml_node_dummy_seq'')];
+		SELECT INTO V_PARENT id
+			FROM xml_node WHERE docid = V_DOCID AND kind = getNodeType(''DUMMY'');
+		-- inserts a node.
+		INSERT INTO xml_node(id,kind,docid,pathid,dewey,parent,value)
+			VALUES(V_NODEID,getNodeType(''TEXT''),V_DOCID,V_NODEID,V_DEWEY,V_PARENT,V_DATANAME);
+		INSERT INTO xml_path VALUES(V_NODEID,''#/text()'',''false'');
+		RAISE DEBUG ''Created PI..%d'',V_NODEID;
+		RETURN V_NODEID;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+
 /**
  -- Document.getElementById(elementId)
  */
@@ -403,9 +620,57 @@ CREATE OR REPLACE FUNCTION hasChildNodes (INTEGER) RETURNS BOOLEAN AS '
 -- not yet supported. to be supported in a feature.
 
 /**
- -- Node.removeChild(oldChild)
+ -- removeChild(CurrentNode INTEGER, ChildNode INTEGER)
+ 
+ Description: 
+	Removes the child node indicated by oldChild from the list of children, and returns it.
+
+ Parameters:  
+	Node oldChild  -  The node being removed.
+
+ Returns:  
+	Node -  The node removed.
+
+ Exceptions:  
+	DOMException NO_MODIFICATION_ALLOWED_ERR
+    	-> yet not supported.
+	DOMException NOT_FOUND_ERR
+    	Raised if oldChild is not a child of this node. 
  */
--- not yet supported.
+/*
+CREATE OR REPLACE FUNCTION removeChild (INTEGER, INTEGER) RETURNS INTEGER AS '
+	DECLARE
+		V_CURRNODE ALIAS FOR $1;
+		V_CHILD ALIAS FOR $2;
+		V_DOCID INTEGER;
+		V_NEXT INTEGER;
+		V_ISFIRSTCHILD BOOLEAN;
+		V_NEW_DEWEY INTEGER[];
+	BEGIN
+		SELECT INTO V_DOCID, V_NEXT, V_ISFIRSTCHILD 
+			docid, next, (dewey[3] = 1)
+			FROM xml_node WHERE id = V_CHILD AND parent = V_CURRNODE;
+		IF NOT FOUND THEN
+			RAISE EXCEPTION ''Raising a DOMException NOT_FOUND_ERR, oldChild is not a child of this node.'';
+		END IF;
+		IF V_ISFIRSTCHILD THEN
+			-- set parent
+			IF V_NEXT IS NOT NULL THEN
+				UPDATE xml_node SET child = V_NEXT WHERE id = V_CURRNODE;
+			ELSE
+				UPDATE xml_node SET child = NULL WHERE id = V_CURRNODE;
+			END IF;
+		ELSE
+			-- set previous node.
+			UPDATE xml_node SET next = V_NEXT WHERE next = V_CHILD;
+		END IF;
+		-- Supported from PostgreSQL 7.4
+		V_NEW_DEWEY := ARRAY[-1, V_DOCID, nextval(''xml_node_dummy_seq'')];
+		UPDATE xml_node set dewey[1:3] = V_NEW_DEWEY WHERE _int_flagment(dewey, V_OLD_DEWEY);
+		RETURN V_CHILD;
+	END;
+' LANGUAGE 'plpgsql' STRICT;
+*/
 
 /**
  -- Node.replaceChild(newChild, oldChild)
@@ -438,7 +703,7 @@ CREATE OR REPLACE FUNCTION getAttribute (INTEGER, VARCHAR) RETURNS VARCHAR AS '
 	    xml_node
 	WHERE
 	    parent = $1 AND
-	    tagid = get_attid($2) AND
+	    tagid = get_attid($2::VARCHAR) AND
 	    kind = getNodeType(''ATTRIBUTE'')
 ' LANGUAGE 'SQL' STABLE STRICT;
 
@@ -483,6 +748,8 @@ CREATE OR REPLACE FUNCTION getElementsByTagName (INTEGER, VARCHAR) RETURNS SETOF
 	    _int_descendant(xn2.dewey,xn1.dewey) AND
 	    xn2.tagid=get_tagid($2) AND
 	    xn2.kind = getNodeType(''ELEMENT'')
+	ORDER BY
+		xn2.dewey
 ' LANGUAGE 'SQL' STABLE STRICT;
 
 /**
@@ -522,7 +789,7 @@ CREATE OR REPLACE FUNCTION removeAttribute (INTEGER, VARCHAR) RETURNS VOID AS '
 	DELETE FROM
 	    xml_node
 	WHERE
-	    parent = $1 and tagid=get_attid($2) and kind=getNodeType(''ATTRIBUTE'')
+	    parent = $1 AND tagid = get_attid($2) AND kind = getNodeType(''ATTRIBUTE'');
 ' LANGUAGE 'SQL' STRICT;
 
 /**
@@ -728,16 +995,19 @@ CREATE OR REPLACE FUNCTION substringData (INTEGER, INTEGER, INTEGER) RETURNS VAR
  -- getFirstChild(curnode INTEGER)::INTEGER
  */
 CREATE OR REPLACE FUNCTION getFirstChild(INTEGER) RETURNS INTEGER AS '
-	SELECT child
-	FROM xml_node
-	WHERE id = $1;
+	SELECT child FROM xml_node WHERE id = $1;
 ' LANGUAGE 'SQL' STABLE STRICT;
 
 /**
  -- getNextSibling(curnode INTEGER)::INTEGER
  */
 CREATE OR REPLACE FUNCTION getNextSibling(INTEGER) RETURNS INTEGER AS '
-	SELECT next
-	FROM xml_node
-	WHERE id = $1;
+	SELECT next FROM xml_node WHERE id = $1;
+' LANGUAGE 'SQL' STABLE STRICT;
+
+/**
+ -- getParent(curnode INTEGER)::INTEGER
+ */
+CREATE OR REPLACE FUNCTION getParent(INTEGER) RETURNS INTEGER AS '
+	SELECT parent FROM xml_node WHERE id = $1;
 ' LANGUAGE 'SQL' STABLE STRICT;
\ No newline at end of file
