@@ -1,12 +1,7 @@
-BEGIN;
-
-DROP AGGREGATE list(VARCHAR);
-DROP FUNCTION Create_New_Document(VARCHAR);
 DROP FUNCTION XML_TagID(VARCHAR);
 DROP FUNCTION XML_AttID(VARCHAR);
 DROP FUNCTION XML_NSID(VARCHAR);
 DROP FUNCTION XML_PIID(VARCHAR);
-DROP FUNCTION Drop_Document(VARCHAR);
 DROP FUNCTION Get_TagID(VARCHAR);
 DROP FUNCTION Get_AttID(VARCHAR);
 DROP FUNCTION Get_NSID(VARCHAR);
@@ -15,16 +10,14 @@ DROP FUNCTION CreateElement(INTEGER,VARCHAR);
 DROP FUNCTION CreateProcessingInstruction(INTEGER,VARCHAR,VARCHAR);
 DROP FUNCTION CreateTextNode(INTEGER,VARCHAR);
 DROP FUNCTION CreateComment(INTEGER,VARCHAR);
-DROP FUNCTION New_ChildElement(INTEGER,VARCHAR);
-DROP FUNCTION New_ChildPI(INTEGER,VARCHAR,VARCHAR);
-DROP FUNCTION New_ChildText(INTEGER,VARCHAR);
-DROP FUNCTION New_ChildComment(INTEGER,VARCHAR);
+DROP FUNCTION addChildElement(INTEGER,VARCHAR);
+DROP FUNCTION addChildPI(INTEGER,VARCHAR,VARCHAR);
+DROP FUNCTION addChildText(INTEGER,VARCHAR);
+DROP FUNCTION addChildComment(INTEGER,VARCHAR);
 DROP FUNCTION SetAttribute(INTEGER,VARCHAR,VARCHAR);
 DROP FUNCTION SetNamespace(INTEGER,VARCHAR,VARCHAR);
 DROP FUNCTION appendChild(INTEGER,INTEGER);
 DROP FUNCTION insertBefore(INTEGER,INTEGER);
 DROP FUNCTION StringValue(INTEGER);
-DROP FUNCTION cat_list(VARCHAR, VARCHAR);
-DROP AGGREGATE list;
 
-END;
\ No newline at end of file
+DROP LANGUAGE plpgsql CASCADE;
\ No newline at end of file
