@@ -2,7 +2,6 @@
  - XPath.sql,v 1.1 2003/07/17 14:10:45 myui Exp
  */
 
---DROP FUNCTION xpath2sql(varchar);
 CREATE or REPLACE FUNCTION xpath2sql(varchar) RETURNS varchar
   AS '/usr/local/lib/xpsql/XPath.so'
   LANGUAGE 'C' IMMUTABLE STRICT;
@@ -11,10 +10,6 @@ CREATE or REPLACE FUNCTION xpath2sql(varchar,integer) RETURNS varchar
   AS '/usr/local/lib/xpsql/XPath.so'
   LANGUAGE 'C' IMMUTABLE STRICT;
 
-DROP FUNCTION xpath_eval(varchar);
-DROP FUNCTION xpath_eval(varchar,integer);
-
-DROP TYPE __retxpatheval;
 CREATE TYPE __retxpatheval AS (docid integer, id integer);
 
 CREATE or REPLACE FUNCTION xpath_eval(varchar) RETURNS setof __retxpatheval
@@ -33,8 +28,7 @@ CREATE or REPLACE FUNCTION vchar2real(VARCHAR) RETURNS REAL
   AS '/usr/local/lib/xpsql/XPath.so'
   LANGUAGE 'C' IMMUTABLE STRICT;
 
-DROP TYPE __retxpathexec;
-CREATE TYPE __retxpathexec AS (document integer, result varchar);
+CREATE TYPE __retxpathexec AS (docid integer, result varchar);
 
 CREATE or REPLACE FUNCTION xpath_evalx(varchar) RETURNS setof __retxpathexec
   AS '/usr/local/lib/xpsql/XPath.so'
@@ -42,7 +36,7 @@ CREATE or REPLACE FUNCTION xpath_evalx(varchar) RETURNS setof __retxpathexec
 
 
 /*
-	-- Exsamples
+	-- Examples
 
 select xpath2sql('/aaa/bbb[2][aaa="bbb"]');
 select * from xpath_eval('/aaa/bbb[2][aaa="bbb"]');
