@@ -5,8 +5,8 @@
 #ifndef __XPSQL__
 #define __XPSQL__
 
-#include "postgres.h"			/* for DatumGetInt32 */
-#include "fmgr.h"				/* for PG_GETARG_DATUM */
+#include "postgres.h"            /* for DatumGetInt32 */
+#include "fmgr.h"                /* for PG_GETARG_DATUM */
 
 // NODE-TYPE and NUMBER BINDING
 #define KIND_ROOT       0
@@ -16,59 +16,64 @@
 #define KIND_PI         4
 #define KIND_ATTRIBUTE  5
 #define KIND_NAMESPACE  6
-#define KIND_DUMMY		-1
+#define KIND_DUMMY        -1
 
 // xpsql Revision Code
 #define __XPSQL_REVISION__ "0.9"
 
+/* Initial buffer size is setted to 8K(8092) byte */
+#define XBUFSIZE 8192
+
 #define SUCCESS (0==0)
 #define FAILURE (0!=0)
 #define SQL_QUERY_SUCCESS  0
 #define SQL_QUERY_FAILURE -1
 
+#define GET_STR(vcharp) DatumGetCString(DirectFunctionCall1(varcharout, PointerGetDatum(vcharp)))
+
 int exec_query(char *);
-int execp_query(char *, int, Oid *, Datum *, char *, char);
-Datum create_new_document(PG_FUNCTION_ARGS);
+int execp_query(char *, int, Oid *, Datum *, char *);
+static int va_exec_query(char *query_str, ...);
+Datum h_replace(PG_FUNCTION_ARGS);
 Datum xml_tagid(PG_FUNCTION_ARGS);
 Datum xml_attid(PG_FUNCTION_ARGS);
 Datum xml_nsid(PG_FUNCTION_ARGS);
 Datum xml_piid(PG_FUNCTION_ARGS);
-Datum drop_document(PG_FUNCTION_ARGS);
 Datum get_tagid(PG_FUNCTION_ARGS);
 Datum get_attid(PG_FUNCTION_ARGS);
 Datum get_nsid(PG_FUNCTION_ARGS);
 Datum get_piid(PG_FUNCTION_ARGS);
-Datum createelement(PG_FUNCTION_ARGS);
-Datum createprocessinginstruction(PG_FUNCTION_ARGS);
-Datum createtextnode(PG_FUNCTION_ARGS);
-Datum createcomment(PG_FUNCTION_ARGS);
-Datum new_childelement(PG_FUNCTION_ARGS);
-Datum new_childpi(PG_FUNCTION_ARGS);
-Datum new_childtext(PG_FUNCTION_ARGS);
-Datum new_childcomment(PG_FUNCTION_ARGS);
+Datum addchildelement(PG_FUNCTION_ARGS);
+Datum addchildpi(PG_FUNCTION_ARGS);
+Datum addchildtext(PG_FUNCTION_ARGS);
+Datum addchildcomment(PG_FUNCTION_ARGS);
 Datum setattribute(PG_FUNCTION_ARGS);
 Datum setnamespace(PG_FUNCTION_ARGS);
 Datum appendchild(PG_FUNCTION_ARGS);
 Datum insertbefore(PG_FUNCTION_ARGS);
 Datum stringvalue(PG_FUNCTION_ARGS);
 Datum toxml(PG_FUNCTION_ARGS);
-static int StrReplace(char *, const char *, const char *);
+Datum trg_xorder(PG_FUNCTION_ARGS);
 
 typedef struct
 {
-	char	   *ident;	// identifier
-	void	  *splan;	// entity
-	//int nplans; /* for the case argument type is not static. */
-}	EPlan;
+    char *ident;    // identifier
+    void *splan;    // entity
+} EPlan;
 
 /* select plan */
-static EPlan *sPlans = NULL;		// array of plans
-static int	n_sPlans = 0;			// count plans
-/* other plan */
-static EPlan *oPlans = NULL;		// array of plans
-static int	n_oPlans = 0;			// count plans
+static EPlan *sPlans = NULL;        // array of plans
+static int    n_sPlans = 0;         // count plans
 
 static EPlan *find_plan(char *, EPlan **, int *);
 static void create_plan(EPlan **, char *, int, Oid *);
 
+#ifdef _Bool
+#define bool _Bool
+#else
+#ifndef C_H
+typedef char bool;
+#endif//C_H
+#endif//_Bool
+
 #endif/* __XPSQL__ */
